package ballerina.lang.datatables;

@doc:Description ("Retrieves the Boolean value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("name: The column name of the output result." )
@doc:Return ("boolean: The column value as a Boolean" )
native function getBoolean (datatable dt, string name) (boolean);

@doc:Description ("Retrieves the string value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc. " )
@doc:Return ("string: The column value as a string" )
native function getString (datatable dt, int index) (string);

@doc:Description ("Checks for a new row in the given datatable. If a new row is found, moves the cursor to it.")
@doc:Param ("dt: The datatable object" )
@doc:Return ("boolean: True if there is a new row; false otherwise" )
native function next (datatable dt) (boolean);

@doc:Description ("Retrieves the double value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc. " )
@doc:Return ("double: The column value as a double" )
native function getDouble (datatable dt, int index) (double);

@doc:Description ("Retrieves the string value of the designated column in the current row. The value of type blob and binary columns will return as a Base64Encoded string.")
@doc:Param ("dt: The datatable object" )
@doc:Param ("name: The column name of the output result." )
@doc:Return ("string: The column value as a string" )
native function getValueAsString (datatable dt, string name) (string);

@doc:Description ("Retrieves the float value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc. " )
@doc:Return ("float: The column value as a float" )
native function getFloat (datatable dt, int index) (float);

@doc:Description ("Retrieves the long value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc. " )
@doc:Return ("long: The column value as a long" )
native function getLong (datatable dt, int index) (long);

@doc:Description ("Retrieves the string value of the designated column in the current row. The value of type blob and binary columns will return as a Base64Encoded string.")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc. " )
@doc:Return ("string: The column value as a string" )
native function getValueAsString (datatable dt, int index) (string);

@doc:Description ("Retrieves the base64encoded string value of the designated column in the current row for the given column type: blob, clob, nclob, or binary")
@doc:Param ("dt: The datatable object" )
@doc:Param ("string: The column name of the output result." )
@doc:Param ("type: Database table column type. Supported values are blob, clob, nclob, binary." )
@doc:Return ("string: The column value as a string" )
native function getString (datatable dt, string column, string type) (string);

@doc:Description ("Retrieves the integer value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("name: The column name of the output result." )
@doc:Return ("int: The column value as an integer" )
native function getInt (datatable dt, string name) (int);

@doc:Description ("Retrieves the Boolean value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc." )
@doc:Return ("boolean: The column value as a Boolean" )
native function getBoolean (datatable dt, int index) (boolean);

@doc:Description ("Retrieves the long value of the designated column in the current row for the given column type: date, time, or timestamp")
@doc:Param ("dt: The datatable object" )
@doc:Param ("string: The column name of the output result." )
@doc:Param ("type: Database table column type. Supported values are date, time, timestamp" )
@doc:Return ("long: The column value as a long" )
native function getLong (datatable dt, string column, string type) (long);

@doc:Description ("Retrieves arrays values of a given column index.")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: " )
@doc:Return ("map): The column value as map" )
native function getArray (datatable dt, int index) (map);

@doc:Description ("Retrieves the float value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("name: The column name of the output result." )
@doc:Return ("float: The column value as a float" )
native function getFloat (datatable dt, string name) (float);

@doc:Description ("Retrieves the integer value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc. " )
@doc:Return ("int: The column value as an integer" )
native function getInt (datatable dt, int index) (int);

@doc:Description ("Outputs the dataset in JSON format as a stream.")
@doc:Param ("dt: The datatable object" )
@doc:Return ("json: The resulting dataset in JSON format" )
native function toJson (datatable dt) (json);

@doc:Description ("Retrieves the long value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("name: The column name of the output result." )
@doc:Return ("long: The column value as a long" )
native function getLong (datatable dt, string name) (long);

@doc:Description ("Retrieves the base64encoded string value of the designated column in the current row for the given column type: blob, clob, nclob, or binary")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc. " )
@doc:Param ("type: Database table column type. Supported values are blob, clob, nclob, binary." )
@doc:Return ("string: The column value as a string" )
native function getString (datatable dt, int index, string type) (string);

@doc:Description ("Retrieves the long value of the designated column in the current row for the given column type: date, time, or timestamp")
@doc:Param ("dt: The datatable object" )
@doc:Param ("index: The column index position of the result. The first column is 1, the second is 2, etc. " )
@doc:Param ("type: Database table column type. Supported values are date, time, timestamp" )
@doc:Return ("long: The column value as a long" )
native function getLong (datatable dt, int index, string type) (long);

@doc:Description ("Retrieves arrays values of a given column name.")
@doc:Param ("dt: The datatable object" )
@doc:Param ("name: " )
@doc:Return ("map): The column value as map" )
native function getArray (datatable dt, string name) (map);

@doc:Description ("Releases the database connection.")
@doc:Param ("dt: The datatable object" )
native function close (datatable dt);

@doc:Description ("Retrieves the string value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("name: The column name of the output result." )
@doc:Return ("string: The column value as a string" )
native function getString (datatable dt, string name) (string);

@doc:Description ("Outputs the dataset in XML format as a stream. This function will add 'results' and 'result' if the root wrapper and row wrapper elements are not provided. ")
@doc:Param ("dt: The datatable object" )
@doc:Param ("rootWrapper: The root wrapper element" )
@doc:Param ("rowWrapper: The row wrapper element" )
@doc:Return ("xml: The resulting dataset in XML format with given root wrapper and row wrapper. The default will be results and result if the wrapper isn't provided. " )
native function toXml (datatable dt, string rootWrapper, string rowWrapper) (xml);

@doc:Description ("Retrieves the double value of the designated column in the current row")
@doc:Param ("dt: The datatable object" )
@doc:Param ("name: The column name of the output result." )
@doc:Return ("double: The column value as a double" )
native function getDouble (datatable dt, string name) (double);

