package ballerina.net.http;

@doc:Description ("Converts the message into an HTTP response")
@doc:Param ("m: A message object" )
native function convertToResponse (message m);

@doc:Description ("Gets the Content-Length header from the message")
@doc:Param ("m: A message object" )
@doc:Return ("int: length of the message" )
native function getContentLength (message m) (int);

@doc:Description ("Gets the HTTP method from the message")
@doc:Param ("m: A message object" )
@doc:Return ("string: http method value" )
native function getMethod (message m) (string);

@doc:Description ("Gets the request URL from the message")
@doc:Param ("m: The message object" )
@doc:Return ("string: The request URL value" )
native function getRequestURL (message m) (string);

@doc:Description ("Gets the HTTP status code from the message")
@doc:Param ("m: A message object" )
@doc:Return ("int: http status code" )
native function getStatusCode (message m) (int);

@doc:Description ("Sets the Content-Length header on the message")
@doc:Param ("m: A message object" )
@doc:Param ("contentLength: Length of the message" )
native function setContentLength (message m, int contentLength);

@doc:Description ("Sets a custom HTTP Reason phrase")
@doc:Param ("m: A message object" )
@doc:Param ("reasonPhrase: Reason phrase value" )
native function setReasonPhrase (message m, string reasonPhrase);

@doc:Description ("Sets the HTTP StatusCode on the message")
@doc:Param ("m: A message object" )
@doc:Param ("statusCode: HTTP status code" )
native function setStatusCode (message m, int statusCode);

@doc:Description ("Native HTTP Client Connector")
native connector ClientConnector (string serviceUri) {

	@doc:Description ("The DELETE action implementation of the HTTP connector")
	@doc:Param ("c: A connector object" )
	@doc:Param ("path: Resource path " )
	@doc:Param ("m: A message object" )
	@doc:Return ("message: The response message object" )
	native action delete (ClientConnector c, string path, message m) (message);

	@doc:Description ("Invokes an HTTP call with the specified HTTP verb.")
	@doc:Param ("c: A connector object" )
	@doc:Param ("httpVerb: HTTP verb value" )
	@doc:Param ("path: Resource path " )
	@doc:Param ("m: A message object" )
	@doc:Return ("message: The response message object" )
	native action execute (ClientConnector c, string httpVerb, string path, message m) (message);

	@doc:Description ("GET action implementation of the HTTP Connector")
	@doc:Param ("c: Connector" )
	@doc:Param ("path: Request path" )
	@doc:Param ("m: message" )
	native action get (ClientConnector c, string path, message m) (message);

	@doc:Description ("The HEAD action implementation of the HTTP Connector.")
	@doc:Param ("c: A connector object" )
	@doc:Param ("path: Resource path " )
	@doc:Param ("m: A message object" )
	@doc:Return ("message: The response message object" )
	native action head (ClientConnector c, string path, message m) (message);

	@doc:Description ("The PATCH action implementation of the HTTP Connector.")
	@doc:Param ("c: A connector object" )
	@doc:Param ("path: Resource path " )
	@doc:Param ("m: A message object" )
	@doc:Return ("message: The response message object" )
	native action patch (ClientConnector c, string path, message m) (message);

	@doc:Description ("The POST action implementation of the HTTP Connector.")
	@doc:Param ("c: A connector object" )
	@doc:Param ("path: Resource path " )
	@doc:Param ("m: A message object" )
	@doc:Return ("message: The response message object" )
	native action post (ClientConnector c, string path, message m) (message);

	@doc:Description ("The PUT action implementation of the HTTP Connector.")
	@doc:Param ("c: A connector object" )
	@doc:Param ("path: Resource path " )
	@doc:Param ("m: A message object" )
	@doc:Return ("message: The response message object" )
	native action put (ClientConnector c, string path, message m) (message);

}